@@ -1,7 +1,7 @@
 package Dist::Zilla::Plugin::GenPericmdScript;
 
-our $DATE = '2015-07-08'; # DATE
-our $VERSION = '0.18'; # VERSION
+our $DATE = '2015-07-10'; # DATE
+our $VERSION = '0.19'; # VERSION
 
 use 5.010001;
 use strict;
@@ -9,6 +9,9 @@ use warnings;
 
 use Moose;
 with (
+    'Dist::Zilla::Role::FileFinderUser' => {
+        default_finders => [':InstallModules'],
+    },
     'Dist::Zilla::Role::FileGatherer',
     'Dist::Zilla::Role::FileMunger',
     'Dist::Zilla::Role::PERLANCAR::WriteModules',
@@ -42,6 +45,27 @@ sub gather_files {
     # FileGatherer plugin too to get add_file().
 }
 
+# XXX extract list_own_modules, is_own_module to its own role/dist
+sub is_own_module {
+    use experimental 'smartmatch';
+
+    my ($self, $mod) = @_;
+
+    state $own_modules = do {
+        my @list;
+        for my $file (@{ $self->found_files }) {
+            my $name = $file->name;
+            next unless $name =~ s!^lib[/\\]!!;
+            $name =~ s![/\\]!::!g;
+            $name =~ s/\.(pm|pod)$//;
+            push @list, $name;
+        }
+        \@list;
+    };
+
+    $mod ~~ @$own_modules ? 1:0;
+}
+
 sub munge_files {
     my ($self, $arg) = @_;
 
@@ -103,10 +127,32 @@ sub munge_files {
 
     {
         my $ver = 0;
-        $self->zilla->register_prereqs(
-            {phase => 'runtime'}, $res->[3]{'func.cmdline_module'} =>
-                $res->[3]{'func.cmdline_module_version'})
-            unless $res->[3]{'func.cmdline_module_inlined'};
+        my %mem;
+        my $perimod = $res->[3]{'func.cmdline_module'};
+        unless ($res->[3]{'func.cmdline_module_inlined'}) {
+            $self->log_debug(["Adding prereq to %s", $perimod]);
+            $self->zilla->register_prereqs(
+                {phase => 'runtime'}, $perimod =>
+                    $res->[3]{'func.cmdline_module_version'});
+            $mem{$perimod}++;
+        }
+        my @urls = ($self->url);
+        if ($subcommands && @$subcommands) {
+            for my $sc (@$subcommands) {
+                /:(.+)/;
+                push @urls, $1;
+            }
+        }
+        # add prereq to script backend modules
+        for my $url (@urls) {
+            my ($pkg) = $url =~ m!^(?:pm:)?/(.+)/.*!;
+            next unless $pkg;
+            $pkg =~ s!/!::!g;
+            next if $self->is_own_module($pkg);
+            next if $mem{$pkg}++;
+            $self->log_debug(["Adding prereq to %s", $pkg]);
+            $self->zilla->register_prereqs({phase => 'runtime'}, $pkg => 0);
+        }
     }
 
     my $fileobj = Dist::Zilla::File::InMemory->new(
@@ -132,7 +178,7 @@ Dist::Zilla::Plugin::GenPericmdScript - Generate Perinci::CmdLine script
 
 =head1 VERSION
 
-This document describes version 0.18 of Dist::Zilla::Plugin::GenPericmdScript (from Perl distribution Dist-Zilla-Plugin-GenPericmdScript), released on 2015-07-08.
+This document describes version 0.19 of Dist::Zilla::Plugin::GenPericmdScript (from Perl distribution Dist-Zilla-Plugin-GenPericmdScript), released on 2015-07-10.
 
 =head1 SYNOPSIS
 
